#ifndef __AGENTS_H__
#define __AGENTS_H__

#include "agents.h"

/*********************** Motor Agent *************************/

void MotorAgent::init(string args) {
    ConfBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.jointBlock = "head_joints";
    defConf.accelerationBlock = "head_acc";
    defConf.speedBlock = "head_speed";
    defConf.yarpDevice = "remote_controlboard";
    defConf.yarpLocal = "/test/client";
    defConf.yarpRemote = "/icubSim/head";


    conf = space_read(args, defConf);

    yarp::os::Property options;
    options.put("device", conf.yarpDevice);
    options.put("local", conf.yarpLocal);
    options.put("remote", conf.yarpRemote);

    driver.open(options);
    driver.view(encs);
    //driver.view(vel);
    driver.view(pos);
    driver.view(iLim);

    if ( (!encs) || (!pos) )
        return;

    int joints;
    encs->getAxes(&joints);

    encoders.resize(joints);
    commands.resize(joints);
    speeds.resize(joints);
    accelerations.resize(joints);
    def_commands.resize(joints);
    def_speeds.resize(joints);

    def_commands = 0;
    def_speeds = 30;
    pos->setRefAccelerations(def_speeds.data());
    pos->setRefSpeeds(def_speeds.data());


    trigger_attach(conf.jointBlock);
    trigger_attach(conf.accelerationBlock);
    trigger_attach(conf.speedBlock);

}

void MotorAgent::sense_select_act(int pid) {
    min = 0;
    max = 0;
   /*for (int p = 0; p < commands.size(); p++) {
        iLim->getLimits(p, &min, &max);
        printf("limit joint: %d = %f - %f \n",p , max, min);
    }*/
    commands = space_read(conf.jointBlock, def_commands);
    pos->positionMove(commands.data());
   // vel->velocityMove(commands.data());
    string s = "";
    for (int i = 0; i < commands.size(); i++) {
        ostringstream convert;
        convert << commands[i];
        s += " " + convert.str();
    }
    printf("read: %s \n", s.c_str());

    /*encs->getEncoders(encoders.data());
    s = "";
    for (int i = 0; i < encoders.size(); i++) {
        ostringstream convert2;
        convert2 << encoders[i];
        s += " " + convert2.str();
    }
    printf("encoders: %s \n", s.c_str());*/


    accelerations = space_read(conf.accelerationBlock, def_speeds);
    pos->setRefAccelerations(accelerations.data());
    speeds = space_read(conf.speedBlock, def_speeds);
    pos->setRefSpeeds(speeds.data());

}

MotorAgent::MotorAgent(string confBlock) :
        Agent(confBlock) {
    printf("New agent with confBlock %s created \n", confBlock.c_str());
}

MotorAgent::~MotorAgent(){
    pos->stop();
    driver.close();
}

/*********************** Velovity Motor Agent *************************/

void VelocityMotorAgent::init(string args) {
    VelocityBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.velocityBlock = "head_vel";
    defConf.encodersBlock = "head_enc";
    defConf.yarpDevice = "remote_controlboard";
    defConf.yarpLocal = "/test/client2";
    defConf.yarpRemote = "/icubSim/head";

    conf = space_read(args, defConf);

    yarp::os::Property options;
    options.put("device", conf.yarpDevice);
    options.put("local", conf.yarpLocal);
    options.put("remote", conf.yarpRemote);

    driver.open(options);
    driver.view(encs);
    driver.view(vel);

    if ( (!encs) || (!vel) )
        return;

    int joints;
    encs->getAxes(&joints);

    encoders.resize(joints);
    commands.resize(joints);

    def_commands.resize(joints);
    def_commands = 0;

    trigger_attach(conf.velocityBlock);
}

void VelocityMotorAgent::sense_select_act(int pid) {

    commands = space_read(conf.velocityBlock, def_commands);
    vel->velocityMove(commands.data());
    string s = "";
    for (int i = 0; i < commands.size(); i++) {
        ostringstream convert;
        convert << commands[i];
        s += " " + convert.str();
    }
    printf("read: %s \n", s.c_str());

    /*encs->getEncoders(encoders.data());
    s = "";
    for (int i = 0; i < encoders.size(); i++) {
        ostringstream convert2;
        convert2 << encoders[i];
        s += " " + convert2.str();
    }
    printf("encoders: %s \n", s.c_str());*/

}

VelocityMotorAgent::VelocityMotorAgent(string confBlock) :
        Agent(confBlock) {
    printf("New agent with confBlock %s created \n", confBlock.c_str());
}

VelocityMotorAgent::~VelocityMotorAgent(){
    vel->stop();
    driver.close();
}

/*********************** Position Control Agent *************************/

void PositionAgent::init(string args) {
    PositionBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.encodersBlock = "head_enc";
    defConf.yarpDevice = "remote_controlboard";
    defConf.yarpLocal = "/test/client2";
    defConf.yarpRemote = "/icubSim/head";
    defConf.startTime = 100;
    defConf.period = 500;

    conf = space_read(args, defConf);

    yarp::os::Property options;
    options.put("device", conf.yarpDevice);
    options.put("local", conf.yarpLocal);
    options.put("remote", conf.yarpRemote);

    driver.open(options);
    driver.view(encs);

    if ( !encs )
        return;

    int joints;
    encs->getAxes(&joints);
    encoders.resize(joints);
    timer_attach(conf.startTime,conf.period);
}

void PositionAgent::sense_select_act(int pid) {
    string s = "";
    encs->getEncoders(encoders.data());
    for (int i = 0; i < encoders.size(); i++) {
        ostringstream convert2;
        convert2 << encoders[i];
        s += " " + convert2.str();
    }
   //printf("encoders: %s \n", s.c_str());
    space_write(conf.encodersBlock, encoders);

}

PositionAgent::PositionAgent(string confBlock) :
        Agent(confBlock) {
    printf("New agent with confBlock %s created \n", confBlock.c_str());
}

PositionAgent::~PositionAgent(){
    driver.close();
}

/*********************** Camera Sensor Agent *************************/

void CamSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.deviceName = "/icubSim/cam/left";
    defConf.portName = "/left/image/in";
    defConf.outBlock = "leftImage";
    defConf.startTime = 1000;
    defConf.period = 250;

    conf = space_read(args, defConf);

    imagePort = new  yarp::os::BufferedPort<yarp::sig::ImageOf<yarp::sig::PixelRgb> >();

    imagePort->open(conf.portName);
    yarp::os::Network::connect(conf.deviceName,conf.portName);

    timer_attach(conf.startTime, conf.period);
}

void CamSensorAgent::sense_select_act(int pid) {
    yarp::sig::ImageOf<yarp::sig::PixelRgb> *image = imagePort->read();  // read an image
    if (image!=NULL) {
        //printf("Image %dx%d\n", image->width(), image->height());
        space_write(conf.outBlock, image);
        //PixelRgb& pixel = image->pixel(100,100);
        //printf("red value %d\n", pixel.r);
    } else {
        printf("No image\n");
    }

}

CamSensorAgent::CamSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

CamSensorAgent::~CamSensorAgent(){
    imagePort->close();
}
/******************** Hand position Agent ********************/
void HandPosSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.deviceName = "/icubSim/inertial";
    defConf.portName = "/lhand/pos";
    defConf.outBlock = "lhand_pos";
    defConf.message = "world get lhand";
    defConf.startTime = 1000;
    defConf.period = 500;

    conf = space_read(args, defConf);

    port = new yarp::os::Port();
    port->open(conf.portName);
    yarp::os::Network::connect(conf.portName, conf.deviceName);

    timer_attach(conf.startTime, conf.period);
}

void HandPosSensorAgent::sense_select_act(int pid) {
    yarp::os::Bottle message(conf.message);
    port->write(message,reply);
    position.resize(reply.size());
    for (int i = 0; i < reply.size(); i++) {
        position[i] = reply.get(i).asDouble();
        printf("Position %f \n", position[i]);
    }
    space_write(conf.outBlock, position);
}

HandPosSensorAgent::HandPosSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

HandPosSensorAgent::~HandPosSensorAgent(){
    port->close();
}

/******************** Inertial Senzor Agent ********************/

void InertialSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "InertialAgent";
    defConf.deviceName = "/icubSim/inertial";
    defConf.portName = "/inertial";
    defConf.outBlock = "inertial";
    defConf.startTime = 1000;
    defConf.period = 500;

    conf = space_read(args, defConf);

    port = new yarp::os::BufferedPort<yarp::sig::Vector>();
    port->open(conf.portName);
    yarp::os::Network::connect(conf.deviceName,conf.portName);

    timer_attach(conf.startTime, conf.period);
}

void InertialSensorAgent::sense_select_act(int pid) {
    yarp::sig::Vector *tmp = port->read();
    respond = *tmp;
    for (int i = 0; i < tmp->size(); i++) {
        printf("position %f \n", respond[i]);
    }
    space_write(conf.outBlock, respond);
}

InertialSensorAgent::InertialSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

InertialSensorAgent::~InertialSensorAgent(){
    port->close();
}

/******************** Touch Senzor Agent ********************/

void TouchSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "TouchAgent";
    defConf.deviceName = "/icubSim/skin/left_hand";
    defConf.portName = "/skin/left_touch";
    defConf.outBlock = "left_touch";
    defConf.startTime = 1000;
    defConf.period = 500;

    conf = space_read(args, defConf);

    port = new yarp::os::BufferedPort<yarp::sig::Vector>();
    port->open(conf.portName);
    yarp::os::Network::connect(conf.deviceName,conf.portName);

    timer_attach(conf.startTime, conf.period);
}

void TouchSensorAgent::sense_select_act(int pid) {
    yarp::sig::Vector *tmp = port->read();
    respond = *tmp;
     //printf("size %d \n", tmp->size());
   // for (int i = 0; i < tmp->size(); i++) {
    //    printf("position %f \n", respond[i]);
   // }
    space_write(conf.outBlock, respond);
}

TouchSensorAgent::TouchSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

TouchSensorAgent::~TouchSensorAgent(){
    port->close();
}

/************************ BallPos Agent *****************************/

void BallPosAgent::init(string args) {
    AgentBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.inBlock = "leftImage";
    defConf.outBlock = "ball_pos";

    conf = space_read(args, defConf);

    trigger_attach(conf.inBlock);
}

void BallPosAgent::sense_select_act(int pid) {
    yarp::sig::ImageOf<yarp::sig::PixelRgb> *defimage = NULL;
    yarp::sig::ImageOf<yarp::sig::PixelRgb> *image = space_read(conf.inBlock, defimage);
    yarp::sig::Vector target;
    target.resize(5);
    double xMean = 0;
    double yMean = 0;
    double count = 0;
    int width = 0;
    int height = 0;

    if (image != NULL) {
        width = image->width();
        height = image->height();
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                yarp::sig::PixelRgb& pixel = image->pixel(x,y);
                if (pixel.b > pixel.r*1.2+10 && pixel.b > pixel.g*1.2+10) {
                    xMean += x;
                    yMean += y;
                    count++;
                }
            }
        }
        if (count > 0) {
            xMean /= count;
            yMean /= count;
        }
        if (count > (image->width()/20)*(image->height()/20)) {
            printf("Best guess at blue target: %g %g\n", xMean, yMean);
            target[0] = xMean;
            target[1] = yMean;
            target[2] = width;
            target[3] = height;
            target[4] = 1;
        } else {
            target[0] = 0;
            target[1] = 0;
            target[2] = width;
            target[3] = height;
            target[4] = 0;
        }

    } else {
        printf("No Image \n");
        target[0] = 0;
        target[1] = 0;
        target[2] = width;
        target[3] = height;
        target[4] = 0;
    }
    space_write(conf.outBlock, target);

}

BallPosAgent::BallPosAgent(string agentBlock) :
        Agent(agentBlock) {
    printf("New agent with confBlock %s created \n", agentBlock.c_str());
}

BallPosAgent::~BallPosAgent(){

}

/************************ LookAt Agent *****************************/

void LookAtAgent::init(string args) {
    AgentBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.inBlock = "ball_pos";
    defConf.outBlock = "head_vel";

    conf = space_read(args, defConf);

    def_command.resize(6);
    def_target.resize(5);
    def_command = 0;
    def_target = 0;

    trigger_attach(conf.inBlock);
}

void LookAtAgent::sense_select_act(int pid) {
    commands = space_read(conf.outBlock, def_command);
    //commands = def_command;
    target = space_read(conf.inBlock, def_target);
    printf("target: ");
    for (int i=0; i<target.size(); i++) {
     printf(" %g", target[i]);
    }
    printf("\n");
    if (target[4] > 0.5) {
        //commands = def_command;
        commands[0] = -(target[1] - target[3]/2) * 0.4;
        commands[2] = -(target[0] - target[2]/2) * 0.4;
        //commands[4] = commands[2];
        space_write(conf.outBlock, commands, 300, 100);
    } else {
        commands[0] = 0;
        commands[2] = 0;
        //space_write(conf.outBlock, commands, 0, 0.5);
    }

}

LookAtAgent::LookAtAgent(string agentBlock) :
        Agent(agentBlock) {
    printf("New agent with confBlock %s created \n", agentBlock.c_str());
}

LookAtAgent::~LookAtAgent(){

}
/************************ Rest Pos Agent *****************************/

void RestPosAgent::init(string args) {
    AgentBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.inBlock = "ball_pos";
    defConf.outBlock = "head_vel";
    defConf.posBlock = "head_enc";
    defConf.startTime = 1500;
    defConf.period = 2500;

    conf = space_read(args, defConf);

    def_command.resize(6);
    def_enc.resize(6);
    def_command = 0;
    def_enc = 0;


    timer_attach(conf.startTime, conf.period);
}

void RestPosAgent::sense_select_act(int pid) {
    enc = space_read(conf.posBlock, def_enc);
    commands = space_read(conf.outBlock, def_command);

    if (enc[0] > 0) {
        commands[0] = -(enc[0] * 0.3);
    } else {
        commands[0] = -(enc[0] * 0.3);
    }
    if (enc[2] > 0) {
        commands[2] = -(enc[2] * 0.3);
    } else {
        commands[2] = -(enc[2] * 0.3);
    }
    space_write(conf.outBlock, commands, 300, 1);


}

RestPosAgent::RestPosAgent(string agentBlock) :
        Agent(agentBlock) {
    printf("New agent with confBlock %s created \n", agentBlock.c_str());
}

RestPosAgent::~RestPosAgent(){

}

/************************ Focus Agent *****************************/

void FocusAgent::init(string args) {
    AgentBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.inBlock = "ball_pos";
    defConf.outBlock = "head_vel";
    defConf.posBlock = "head_enc";

    conf = space_read(args, defConf);

    def_command.resize(6);
    def_enc.resize(6);
    def_target.resize(5);
    def_command = 0;
    def_enc = 0;
    def_target = 0;

    //timer_attach(800, 500);
    trigger_attach(conf.inBlock);
}

void FocusAgent::sense_select_act(int pid) {
    enc = space_read(conf.posBlock, def_enc);
    commands = space_read(conf.outBlock, def_command);
    target = space_read(conf.inBlock, def_target);
    printf("Focus encoder4: %f \n", enc[4]);
     printf("Focus target: %f \n", target[4]);
    if (target[4] > 0.5) {
        if (enc[4] > 0) {
            commands[4] = -(enc[4] * 0.4);
        } else {
            commands[4] = -(enc[4] * 0.4);
        }
        if (enc[3] > 0) {
            commands[3] = -(enc[3] * 0.4);
        } else {
            commands[3] = -(enc[3] * 0.4);
        }
        space_write(conf.outBlock, commands, 300, 100);
    }

}

FocusAgent::FocusAgent(string agentBlock) :
        Agent(agentBlock) {
    printf("New agent with confBlock %s created \n", agentBlock.c_str());
}

FocusAgent::~FocusAgent(){

}

/************************ Eye Version Agent *****************************/

void EyeVersionAgent::init(string args) {
    AgentBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.inBlock = "head_enc";
    defConf.outBlock = "head_vel";
    defConf.startTime = 1000;
    defConf.period = 1200;

    conf = space_read(args, defConf);

    def_command.resize(6);
    def_enc.resize(6);
    def_command = 0;
    def_enc = 0;
    i = 0;

    timer_attach(conf.startTime,conf.period);
}

void EyeVersionAgent::sense_select_act(int pid) {
    enc = space_read(conf.inBlock, def_enc);
    commands = def_command;
    printf("encoder4: %f \n", enc[4]);
    if (enc[4] > 0) {
        commands[4] = -70;
    } else {
        commands[4] = 70;
    }
    space_write(conf.outBlock, commands, 1000, 1);

}

EyeVersionAgent::EyeVersionAgent(string agentBlock) :
        Agent(agentBlock) {
    printf("New agent with confBlock %s created \n", agentBlock.c_str());
}

EyeVersionAgent::~EyeVersionAgent(){

}

/************************ Eye Tilt Agent *****************************/

void EyeTiltAgent::init(string args) {
    AgentBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.inBlock = "head_vel";
    defConf.outBlock = "head_vel";
    defConf.startTime = 1000;
    defConf.period = 2000;

    conf = space_read(args, defConf);

    def_command.resize(6);
    def_enc.resize(6);
    def_command = 0;
    def_enc = 0;

    timer_attach(conf.startTime,conf.period);
}

void EyeTiltAgent::sense_select_act(int pid) {
    enc = space_read(conf.inBlock, def_enc);
    commands = def_command;
    if (enc[3] > 0) {
        commands[3] = -70;
    } else {
        commands[3] = 70;
    }
    space_write(conf.outBlock, commands, 250, 10);

}

EyeTiltAgent::EyeTiltAgent(string agentBlock) :
        Agent(agentBlock) {
    printf("New agent with confBlock %s created \n", agentBlock.c_str());
}

EyeTiltAgent::~EyeTiltAgent(){

}

/************************ Helper agent for testing ************************/
void WriteAgent::init(string args) {
    i = 0;
    timer_attach(1000,1000);
    commands.resize(6);
    commands = 0;

}

void WriteAgent::sense_select_act(int pid) {
    if(i == 0) {
        space_write("head_joints", commands);
        commands[2] = 40;
        i = 1;
    } else {
        space_write("head_joints", commands);
        commands[2] = 0;
        i = 0;
    }

}

WriteAgent::WriteAgent(string args) :
        Agent(args) {
     printf("Agent %s created \n", args.c_str());
}

WriteAgent::~WriteAgent(){}

#endif
