#include "agentspace/agentspace.h"

#include <QtGui/QApplication>
#include "headcontrolgui.h"
#include "armcontrolgui.h"
#include "worldgui.h"

#include <yarp/os/all.h>
#include <yarp/sig/all.h>
#include <yarp/dev/all.h>

#include <string>
#include <iostream>
#include <sstream>

/******************** Config Block ********************/
typedef struct {
    string agentName;
    string jointBlock;
    string speedBlock;
    string accelerationBlock;
    string yarpDevice;
    string yarpLocal;
    string yarpRemote;

} ConfBlock;

typedef struct {
    string agentName;
    string velocityBlock;
    string encodersBlock;
    string yarpDevice;
    string yarpLocal;
    string yarpRemote;

} VelocityBlock;

typedef struct {
    string agentName;
    string encodersBlock;
    string yarpDevice;
    string yarpLocal;
    string yarpRemote;
    int startTime;
    int period;

} PositionBlock;

typedef struct {
    string agentName;
    string portName;
    string deviceName;
    string outBlock;
    string message;
    int startTime;
    int period;

} SensorBlock;

typedef struct {
    string agentName;
    string inBlock;
    string outBlock;
    string posBlock;
    int startTime;
    int period;

} AgentBlock;


/******************** Motor Agent ********************/

class MotorAgent : public Agent {
    private:
        yarp::dev::PolyDriver driver;
        yarp::dev::IPositionControl *pos;
        yarp::dev::IVelocityControl *vel;
        yarp::dev::IEncoders *encs;
        yarp::dev::IControlLimits *iLim;
        yarp::sig::Vector encoders;
        yarp::sig::Vector commands;
        yarp::sig::Vector speeds;
        yarp::sig::Vector accelerations;
        yarp::sig::Vector def_commands;
        yarp::sig::Vector def_speeds;
        ConfBlock conf;
        double min;
        double max;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        MotorAgent(string confBlock);
        ~MotorAgent();
};

/******************** Velovity Motor Agent ********************/

class VelocityMotorAgent : public Agent {
    private:
        yarp::dev::PolyDriver driver;
        yarp::dev::IVelocityControl *vel;
        yarp::dev::IEncoders *encs;
        yarp::sig::Vector encoders;
        yarp::sig::Vector commands;
        yarp::sig::Vector def_commands;
        VelocityBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        VelocityMotorAgent(string velocityBlock);
        ~VelocityMotorAgent();
};

/******************** Position Control Agent ********************/

class PositionAgent : public Agent {
    private:
        yarp::dev::PolyDriver driver;
        yarp::dev::IEncoders *encs;
        yarp::sig::Vector encoders;
        PositionBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        PositionAgent(string confBlock);
        ~PositionAgent();
};

/******************** Camera Senzor Agent ********************/

class CamSensorAgent : public Agent {
    private:
        yarp::os::BufferedPort<yarp::sig::ImageOf<yarp::sig::PixelRgb> > *imagePort;
        SensorBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        CamSensorAgent(string sensorBlock);
        ~CamSensorAgent();
};

/******************** Hand Position Agent ********************/

class HandPosSensorAgent : public Agent {
    private:
        yarp::os::Bottle reply;
        yarp::os::Port *port;
        yarp::sig::Vector position;
        SensorBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        HandPosSensorAgent(string sensorBlock);
        ~HandPosSensorAgent();
};

/******************** Intertial Senzor Agent ********************/

class InertialSensorAgent : public Agent {
private:
        yarp::os::BufferedPort<yarp::sig::Vector> *port;
        yarp::sig::Vector respond;
        SensorBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        InertialSensorAgent(string sensorBlock);
        ~InertialSensorAgent();
};

/******************** Touch Senzor Agent ********************/

class TouchSensorAgent : public Agent {
private:
        yarp::os::BufferedPort<yarp::sig::Vector> *port;
        yarp::sig::Vector respond;
        SensorBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        TouchSensorAgent(string sensorBlock);
        ~TouchSensorAgent();
};


/************************ BallPos Agent *****************************/

class BallPosAgent : public Agent {
    private:
        yarp::sig::Vector commands;
        AgentBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        BallPosAgent(string agentBlock);
        ~BallPosAgent();
};

/************************ LookAt Agent *****************************/

class LookAtAgent : public Agent {
    private:
        yarp::sig::Vector commands;
        yarp::sig::Vector def_command;
        yarp::sig::Vector def_target;
        yarp::sig::Vector target;
        AgentBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        LookAtAgent(string agentBlock);
        ~LookAtAgent();
};

/************************ Rest Pos Agent *****************************/

class RestPosAgent : public Agent {
    private:
        yarp::sig::Vector commands;
        yarp::sig::Vector def_command;
        yarp::sig::Vector def_enc;
        yarp::sig::Vector enc;
        AgentBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        RestPosAgent(string agentBlock);
        ~RestPosAgent();
};

/************************ Focus Agent *****************************/

class FocusAgent : public Agent {
    private:
        yarp::sig::Vector commands;
        yarp::sig::Vector def_command;
        yarp::sig::Vector def_enc;
        yarp::sig::Vector enc;
        yarp::sig::Vector def_target;
        yarp::sig::Vector target;
        AgentBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        FocusAgent(string agentBlock);
        ~FocusAgent();
};

/************************ Eye Version Agent *****************************/

class EyeVersionAgent : public Agent {
    private:
        yarp::sig::Vector commands;
        yarp::sig::Vector def_command;
        yarp::sig::Vector def_enc;
        yarp::sig::Vector enc;
        AgentBlock conf;
        int i;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        EyeVersionAgent(string agentBlock);
        ~EyeVersionAgent();
};

/************************ Eye Tilt Agent *****************************/

class EyeTiltAgent : public Agent {
    private:
        yarp::sig::Vector commands;
        yarp::sig::Vector def_command;
        yarp::sig::Vector def_enc;
        yarp::sig::Vector enc;
        AgentBlock conf;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        EyeTiltAgent(string agentBlock);
        ~EyeTiltAgent();
};

/************************** Helper agent for testing *******************/
class WriteAgent : public Agent {
    private:
        int i;
        yarp::sig::Vector commands;

    protected:
        void init (string args);
        void sense_select_act(int pid);
     public:
        WriteAgent(string args);
        ~WriteAgent();
};
