#include "agents.h"


int main(int argc, char *argv[])
{
    printf("Main started \n");

 /********************** CONFIGURATION *****************************/
    ConfBlock headconf;
    headconf.agentName = "MotorAgent Head";
    headconf.jointBlock = "head_joints";
    headconf.accelerationBlock = "head_acc";
    headconf.speedBlock = "head_speed";
    headconf.yarpDevice = "remote_controlboard";
    headconf.yarpLocal = "/test/clientH";
    headconf.yarpRemote = "/icubSim/head";
    Space::getSpace()->write<ConfBlock>("hConfig",headconf, 0, 1.0);

    VelocityBlock headconfvel;
    headconfvel.agentName = "MotorAgent Head";
    headconfvel.velocityBlock = "head_vel";
    headconfvel.encodersBlock = "head_enc";
    headconfvel.yarpDevice = "remote_controlboard";
    headconfvel.yarpLocal = "/test/clientHV";
    headconfvel.yarpRemote = "/icubSim/head";
    Space::getSpace()->write<VelocityBlock>("hvConfig",headconfvel, 0, 1.0);

    ConfBlock armconf;
    armconf.agentName = "MotorAgent LArm";
    armconf.jointBlock = "la_joints";
    armconf.accelerationBlock = "la_acc";
    armconf.speedBlock = "la_speed";
    armconf.yarpDevice = "remote_controlboard";
    armconf.yarpLocal = "/test/clientLA";
    armconf.yarpRemote = "/icubSim/left_arm";
    Space::getSpace()->write<ConfBlock>("laConfig",armconf, 0, 1.0);

    VelocityBlock armconfvel;
    armconfvel.agentName = "MotorAgent left arm";
    armconfvel.velocityBlock = "larm_vel";
    armconfvel.encodersBlock = "larm_enc";
    armconfvel.yarpDevice = "remote_controlboard";
    armconfvel.yarpLocal = "/test/clientLAV";
    armconfvel.yarpRemote = "/icubSim/left_arm";
    Space::getSpace()->write<VelocityBlock>("lavConfig",armconfvel, 0, 1.0);

    ConfBlock rarmconf;
    rarmconf.agentName = "MotorAgent RArm";
    rarmconf.jointBlock = "ra_joints";
    rarmconf.accelerationBlock = "ra_acc";
    rarmconf.speedBlock = "ra_speed";
    rarmconf.yarpDevice = "remote_controlboard";
    rarmconf.yarpLocal = "/test/clientRA";
    rarmconf.yarpRemote = "/icubSim/right_arm";
    Space::getSpace()->write<ConfBlock>("raConfig",rarmconf, 0, 1.0);

    SensorBlock leConf;
    leConf.agentName = "HeadAgent";
    leConf.deviceName = "/icubSim/cam/left";
    leConf.portName = "/left/image/in";
    leConf.outBlock = "leftImage";
    leConf.startTime = 1000;
    leConf.period = 100;
    Space::getSpace()->write<SensorBlock>("leConfig",leConf, 0, 1.0);

    SensorBlock lhandpos;
    lhandpos.agentName = "HeadAgent";
    lhandpos.deviceName = "/icubSim/world";
    lhandpos.portName = "/lhand/pos";
    lhandpos.outBlock = "lhand_pos";
    lhandpos.message = "world get lhand";
    lhandpos.startTime = 1000;
    lhandpos.period = 500;
    Space::getSpace()->write<SensorBlock>("lhandpos",lhandpos, 0, 1.0);

    SensorBlock inertial;
    inertial.agentName = "InertialAgent";
    inertial.deviceName = "/icubSim/inertial";
    inertial.portName = "/inertial";
    inertial.outBlock = "inertial";
    inertial.message = "";
    inertial.startTime = 1000;
    inertial.period = 500;
    Space::getSpace()->write<SensorBlock>("inertialCfg",inertial, 0, 1.0);

    SensorBlock touch;
    touch.agentName = "TouchAgent";
    touch.deviceName = "/icubSim/skin/left_hand";
    touch.portName = "/skin/left_touch";
    touch.outBlock = "left_touch";
    touch.startTime = 1000;
    touch.period = 500;
    Space::getSpace()->write<SensorBlock>("touch",touch, 0, 1.0);

    PositionBlock headpos;
    headpos.agentName = "MotorAgent left arm";
    headpos.encodersBlock = "head_enc";
    headpos.yarpDevice = "remote_controlboard";
    headpos.yarpLocal = "/test/clientHeadP";
    headpos.yarpRemote = "/icubSim/head";
    headpos.startTime = 100;
    headpos.period = 500;
    Space::getSpace()->write<PositionBlock>("hPosConfig",headpos, 0, 1.0);

    AgentBlock ballpos;
    ballpos.agentName = "Find ball";
    ballpos.inBlock = "leftImage";
    ballpos.outBlock = "ballPos";
    Space::getSpace()->write<AgentBlock>("bPConfig",ballpos, 0, 1.0);

    AgentBlock look;
    look.agentName = "Look At";
    look.inBlock = "ballPos";
    look.outBlock = "head_vel";
    look.posBlock = "head_enc";
    Space::getSpace()->write<AgentBlock>("lookConfig",look, 0, 1.0);

    AgentBlock rest;
    rest.agentName = "Look At";
    rest.inBlock = "ballPos";
    rest.outBlock = "head_vel";
    rest.posBlock = "head_enc";
    rest.startTime = 1500;
    rest.period = 10500;
    Space::getSpace()->write<AgentBlock>("restConfig",rest, 0, 1.0);

    AgentBlock search;
    search.agentName = "search";
    search.inBlock = "head_enc";
    search.outBlock = "head_vel";
    search.startTime = 1000;
    search.period = 1200;
    Space::getSpace()->write<AgentBlock>("search",search, 0, 1.0);

    AgentBlock search2;
    search2.agentName = "search2";
    search2.inBlock = "head_enc";
    search2.outBlock = "head_vel";
    search2.startTime = 1000;
    search2.period = 2000;
    Space::getSpace()->write<AgentBlock>("search2",search2, 0, 1.0);


/************************* GUI ****************************/
    yarp::os::Network yarp;

    QApplication a(argc, argv);

    HeadControlGui h;
    h.blockName = headconf.jointBlock;
    h.show();

    HeadControlGui w;
    w.enableVelocity(true);
    w.blockName = headconfvel.velocityBlock;
    //w.show();

    ArmControlGui la;
    la.setName("Left Arm Controls:");
    la.enableVelocity(false);
    la.blockName = armconfvel.velocityBlock;
    //la.show();

    ArmControlGui ra;
    ra.setName("Right Arm Controls:");
    ra.blockName = rarmconf.jointBlock;
    //ra.show();

    WorldGui world;
    world.show();

/*********************** Agents *****************************/


    vector<Agent*> agents;
    agents.push_back(new MotorAgent("hConfig"));
    agents.push_back(new VelocityMotorAgent("hvConfig"));
    //agents.push_back(new VelocityMotorAgent("lavConfig"));
    //agents.push_back(new MotorAgent("raConfig"));
    agents.push_back(new CamSensorAgent("leConfig"));
    agents.push_back(new PositionAgent("hPosConfig"));
    agents.push_back(new BallPosAgent("bPConfig"));
    agents.push_back(new FocusAgent("lookConfig"));
    agents.push_back(new RestPosAgent("restConfig"));
    agents.push_back(new LookAtAgent("lookConfig"));
    agents.push_back(new EyeVersionAgent("search"));
     //agents.push_back(new HandPosSensorAgent("lhandpos"));
     //agents.push_back(new InertialSensorAgent("inertialCfg"));
   // agents.push_back(new TouchSensorAgent("touch"));


    agents.push_back(new EyeTiltAgent("search2"));

    for (int i = 0; i < agents.size(); i++) {
        cout << agents[i]->start();
    }
    /*for (int i = 0; i < agents.size(); i++){
        agents[i]->join();
    }*/
    
    return a.exec();
}
