#ifndef __AGENTS_H__
#define __AGENTS_H__

#include "agents.h"

/*********************** Motor Agent *************************/

void MotorAgent::init(string args) {
    ConfBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.jointBlock = "head_joints";
    defConf.accelerationBlock = "head_acc";
    defConf.speedBlock = "head_speed";
    defConf.yarpDevice = "remote_controlboard";
    defConf.yarpLocal = "/test/client";
    defConf.yarpRemote = "/icubSim/head";


    conf = space_read(args, defConf);

    yarp::os::Property options;
    options.put("device", conf.yarpDevice);
    options.put("local", conf.yarpLocal);
    options.put("remote", conf.yarpRemote);

    driver.open(options);
    driver.view(encs);
    driver.view(pos);
    driver.view(iLim);

    if ( (!encs) || (!pos) )
        return;

    int joints;
    encs->getAxes(&joints);

    encoders.resize(joints);
    commands.resize(joints);
    speeds.resize(joints);
    accelerations.resize(joints);
    def_commands.resize(joints);
    def_speeds.resize(joints);

    def_commands = 0;
    def_speeds = 30;
    pos->setRefAccelerations(def_speeds.data());
    pos->setRefSpeeds(def_speeds.data());


    trigger_attach(conf.jointBlock);
    trigger_attach(conf.accelerationBlock);
    trigger_attach(conf.speedBlock);

}

void MotorAgent::sense_select_act(int pid) {
    min = 0;
    max = 0;
   /*for (int p = 0; p < commands.size(); p++) {
        iLim->getLimits(p, &min, &max);
        printf("limit joint: %d = %f - %f \n",p , max, min);
    }*/
    commands = space_read(conf.jointBlock, def_commands);
    pos->positionMove(commands.data());
    string s = "";
    for (int i = 0; i < commands.size(); i++) {
        ostringstream convert;
        convert << commands[i];
        s += " " + convert.str();
    }
    printf("read: %s \n", s.c_str());

    /*encs->getEncoders(encoders.data());
    s = "";
    for (int i = 0; i < encoders.size(); i++) {
        ostringstream convert2;
        convert2 << encoders[i];
        s += " " + convert2.str();
    }
    printf("encoders: %s \n", s.c_str());*/


    accelerations = space_read(conf.accelerationBlock, def_speeds);
    pos->setRefAccelerations(accelerations.data());
    speeds = space_read(conf.speedBlock, def_speeds);
    pos->setRefSpeeds(speeds.data());

}

MotorAgent::MotorAgent(string confBlock) :
        Agent(confBlock) {
    printf("New agent with confBlock %s created \n", confBlock.c_str());
}

MotorAgent::~MotorAgent(){
    pos->stop();
    driver.close();
}

/*********************** Velovity Motor Agent *************************/

void VelocityMotorAgent::init(string args) {
    VelocityBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.velocityBlock = "head_vel";
    defConf.encodersBlock = "head_enc";
    defConf.yarpDevice = "remote_controlboard";
    defConf.yarpLocal = "/test/client2";
    defConf.yarpRemote = "/icubSim/head";

    conf = space_read(args, defConf);

    yarp::os::Property options;
    options.put("device", conf.yarpDevice);
    options.put("local", conf.yarpLocal);
    options.put("remote", conf.yarpRemote);

    driver.open(options);
    driver.view(encs);
    driver.view(vel);

    if ( (!encs) || (!vel) )
        return;

    int joints;
    encs->getAxes(&joints);

    encoders.resize(joints);
    commands.resize(joints);

    def_commands.resize(joints);
    def_commands = 0;

    trigger_attach(conf.velocityBlock);
}

void VelocityMotorAgent::sense_select_act(int pid) {

    commands = space_read(conf.velocityBlock, def_commands);
    vel->velocityMove(commands.data());
    string s = "";
    for (int i = 0; i < commands.size(); i++) {
        ostringstream convert;
        convert << commands[i];
        s += " " + convert.str();
    }
    printf("read: %s \n", s.c_str());

    /*encs->getEncoders(encoders.data());
    s = "";
    for (int i = 0; i < encoders.size(); i++) {
        ostringstream convert2;
        convert2 << encoders[i];
        s += " " + convert2.str();
    }
    printf("encoders: %s \n", s.c_str());*/

}

VelocityMotorAgent::VelocityMotorAgent(string confBlock) :
        Agent(confBlock) {
    printf("New agent with confBlock %s created \n", confBlock.c_str());
}

VelocityMotorAgent::~VelocityMotorAgent(){
    vel->stop();
    driver.close();
}

/*********************** Position Control Agent *************************/

void PositionAgent::init(string args) {
    PositionBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.encodersBlock = "head_enc";
    defConf.yarpDevice = "remote_controlboard";
    defConf.yarpLocal = "/test/client2";
    defConf.yarpRemote = "/icubSim/head";
    defConf.startTime = 100;
    defConf.period = 500;

    conf = space_read(args, defConf);

    yarp::os::Property options;
    options.put("device", conf.yarpDevice);
    options.put("local", conf.yarpLocal);
    options.put("remote", conf.yarpRemote);

    driver.open(options);
    driver.view(encs);

    if ( !encs )
        return;

    int joints;
    encs->getAxes(&joints);
    encoders.resize(joints);
    timer_attach(conf.startTime,conf.period);
}

void PositionAgent::sense_select_act(int pid) {
    //string s = "";
    encs->getEncoders(encoders.data());
    /*for (int i = 0; i < encoders.size(); i++) {
        ostringstream convert2;
        convert2 << encoders[i];
        s += " " + convert2.str();
    }*/
   //printf("encoders: %s \n", s.c_str());
    space_write(conf.encodersBlock, encoders);

}

PositionAgent::PositionAgent(string confBlock) :
        Agent(confBlock) {
    printf("New agent with confBlock %s created \n", confBlock.c_str());
}

PositionAgent::~PositionAgent(){
    driver.close();
}

/*********************** Camera Sensor Agent *************************/

void CamSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.deviceName = "/icubSim/cam/left";
    defConf.portName = "/left/image/in";
    defConf.outBlock = "leftImage";
    defConf.startTime = 1000;
    defConf.period = 250;

    conf = space_read(args, defConf);

    imagePort = new  yarp::os::BufferedPort<yarp::sig::ImageOf<yarp::sig::PixelRgb> >();

    imagePort->open(conf.portName);
    yarp::os::Network::connect(conf.deviceName,conf.portName);

    timer_attach(conf.startTime, conf.period);
}

void CamSensorAgent::sense_select_act(int pid) {
    yarp::sig::ImageOf<yarp::sig::PixelRgb> *image = imagePort->read();  // read an image
    if (image!=NULL) {
        //printf("Image %dx%d\n", image->width(), image->height());
        space_write(conf.outBlock, image);
    } else {
        printf("No image\n");
    }

}

CamSensorAgent::CamSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

CamSensorAgent::~CamSensorAgent(){
    imagePort->close();
}
/******************** Hand position Agent ********************/
void HandPosSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "HeadAgent";
    defConf.deviceName = "/icubSim/inertial";
    defConf.portName = "/lhand/pos";
    defConf.outBlock = "lhand_pos";
    defConf.message = "world get lhand";
    defConf.startTime = 1000;
    defConf.period = 500;

    conf = space_read(args, defConf);

    port = new yarp::os::Port();
    port->open(conf.portName);
    yarp::os::Network::connect(conf.portName, conf.deviceName);

    timer_attach(conf.startTime, conf.period);
}

void HandPosSensorAgent::sense_select_act(int pid) {
    yarp::os::Bottle message(conf.message);
    port->write(message,reply);
    position.resize(reply.size());
    for (int i = 0; i < reply.size(); i++) {
        position[i] = reply.get(i).asDouble();
        printf("Position %f \n", position[i]);
    }
    space_write(conf.outBlock, position);
}

HandPosSensorAgent::HandPosSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

HandPosSensorAgent::~HandPosSensorAgent(){
    port->close();
}

/******************** Inertial Senzor Agent ********************/

void InertialSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "InertialAgent";
    defConf.deviceName = "/icubSim/inertial";
    defConf.portName = "/inertial";
    defConf.outBlock = "inertial";
    defConf.startTime = 1000;
    defConf.period = 500;

    conf = space_read(args, defConf);

    port = new yarp::os::BufferedPort<yarp::sig::Vector>();
    port->open(conf.portName);
    yarp::os::Network::connect(conf.deviceName,conf.portName);

    timer_attach(conf.startTime, conf.period);
}

void InertialSensorAgent::sense_select_act(int pid) {
    yarp::sig::Vector *tmp = port->read();
    respond = *tmp;
    for (int i = 0; i < tmp->size(); i++) {
        printf("position %f \n", respond[i]);
    }
    space_write(conf.outBlock, respond);
}

InertialSensorAgent::InertialSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

InertialSensorAgent::~InertialSensorAgent(){
    port->close();
}

/******************** Touch Senzor Agent ********************/

void TouchSensorAgent::init(string args) {
    SensorBlock defConf;
    defConf.agentName = "TouchAgent";
    defConf.deviceName = "/icubSim/skin/left_hand";
    defConf.portName = "/skin/left_touch";
    defConf.outBlock = "left_touch";
    defConf.startTime = 1000;
    defConf.period = 500;

    conf = space_read(args, defConf);

    port = new yarp::os::BufferedPort<yarp::sig::Vector>();
    port->open(conf.portName);
    yarp::os::Network::connect(conf.deviceName,conf.portName);

    timer_attach(conf.startTime, conf.period);
}

void TouchSensorAgent::sense_select_act(int pid) {
    yarp::sig::Vector *tmp = port->read();
    respond = *tmp;
     //printf("size %d \n", tmp->size());
   // for (int i = 0; i < tmp->size(); i++) {
    //    printf("position %f \n", respond[i]);
   // }
    space_write(conf.outBlock, respond);
}

TouchSensorAgent::TouchSensorAgent(string sensorBlock) :
        Agent(sensorBlock) {
    printf("New agent with confBlock %s created \n", sensorBlock.c_str());
}

TouchSensorAgent::~TouchSensorAgent(){
    port->close();
}


#endif
